import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class hw2_reorderable extends PApplet {

/*  Urban Agriculture in Seattle with Polar Area Diagram
    Kurt Kaminski
    Algorithmic Visulization, Prof. George Legrady
    MAT 2015
*/



ControlP5 cp5;
CheckBox checkbox;

int chartRadius = 440;
float innerCircleRadius = chartRadius*.33f;
int xPosition = 20;
int pos = 0;

int pickDataDate = 0;
int hoverTextSwitch = -1;
int sortSwitch = 1;

float marginX = 90;
float marginY = 40;    
int [][] dataMatrix = null;
float [] averages = new float[96];
float avgColorPick = 0;
int maxCount;
int numRows, numColumns, rowSpacing;

ArrayList<IntList> sortedYearMonth = new ArrayList<IntList>();  
IntList column_picks; // user selection of column data

String [] deweyNames = new String[10];
String [] monthNames = {  "January", "February", "March", "April", "May", "June", "July", "August",
                          "September", "October", "November", "December"};
String [] yearNames = {"2006","2007","2008","2009","2010","2011","2012","2013"};
PFont fontHeader = createFont("SukhumvitSet-SemiBold", 30, true);
PFont fontHeader2 = createFont("SourceSansPro-Semibold", 14, true);
PFont fontCenter = createFont("AvenirNext-UltraLightItalic", 48, true);
PFont fontHover = createFont("AvenirNext-UltraLight", 32, true);
PFont fontMain = createFont("SourceSansPro-Semibold", 12, true);
PFont fontSubtitle = createFont("Avenir-Regular", 11, true);

int bgColor = color(222,221,217);
int textColor = color(84, 81, 109);
int tickColor = color(151,151,184);
int colorHighlight = 0xff4B7A26;
int [] colorMap = {0xffA7F58C,0xff7CDE91,0xff58C593,0xff3CAC91,0xff2B9289,0xff27787D,0xff29606C,0xff2A4857,0xff263141,0xff1D1D29};
int [] colorMapSelected = {0xff105C65,0xff0E6C73,0xff0B7C80,0xff078D8D,0xff059E99,0xff08AFA5,0xff11C1B0,0xff1DD2BA,0xff2BE4C3,0xff3BF7CB};
int [] colorMapMonth = {0xff823B21,0xff8B471D,0xff925518,0xff976311,0xff99720B,0xff998208,0xff95920F,0xff8FA21D,0xff85B22F,0xff76C244,0xff62D25C,0xff40E177};

int sliderValue = 100;
int sliderTicks1 = 100;
int sliderTicks2 = 30;
int selectedMonth = 1;
int selectedYear = 1;
Slider abc;

Table myTable;

public void setup()
{
  size(1200, 900);
  smooth();
  println("setup done");

  rowSpacing = 54;
  myTable = loadTable("hw1_yearmonth.csv", "header");
  numRows = myTable.getRowCount();
  numColumns = myTable.getColumnCount() - 2;
  dataMatrix = new int[numRows][numColumns];
  maxCount = dataMatrix[0][0];

  for (int i = 0; i < numRows; i++)
  {
    for (int j = 0; j < numColumns; j++) //skipping two columns j=2
    {
      dataMatrix[i][j] = myTable.getInt(i, (j+2));
      // print(dataMatrix[i][j] + " " );
      //            print(i + "," + j + " " );

      if (dataMatrix[i][j] > maxCount)
        maxCount = dataMatrix[i][j];
    }
    // println();
  }

  println("Rows: " + numRows + ", Columns: " + numColumns + ", max: " + maxCount);
  println();
  println();

  deweyNames[0] = "Agriculture & related technology";
  deweyNames[1] = "Apparatus, equipment, materials";
  deweyNames[2] = "Plant injuries, diseases, pests";
  deweyNames[3] = "Field & plantation crops";
  deweyNames[4] = "Orchards, fruits, forestry";
  deweyNames[5] = "Garden crops, horticulture";
  deweyNames[6] = "Animal husbandry";
  deweyNames[7] = "Processing dairy";
  deweyNames[8] = "Insect culture";
  deweyNames[9] = "Hunting, fishing, conservation";
 
 
  ///////////////////////////////////////////////////////////////////////
  ///////// Load arrays with data ///////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////

  background(bgColor);
  noStroke();
  
  int [] sorted = null;
  
  // Create sorted lists for each year/month
  for (int i = 0; i < numRows; i++) {
    IntList data = new IntList();
    for (int j = 0; j < numColumns; j++) {
      data.append(dataMatrix[i][j]);
    }
    sortedYearMonth.add(data);
  }

  // calculate all monthly averages
  for (int i = 0; i < numRows; i++) {
    float avg = 0;  
    IntList getAvg = sortedYearMonth.get(i);
      for (int j = 0; j < numColumns; j++) {
        avg += getAvg.get(j);
      }
    averages[i] = avg/10;
//    println(averages[i]);
  }

  // load dewey class visibility so its on by default
  column_picks = new IntList();
  for (int i = 0; i < numColumns; i++) {
    column_picks.append(i);
  }
  
  ///////////////////////////////////////////////////////////////////////
  ///////// Sliders /////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////
  cp5 = new ControlP5(this);
  cp5.setControlFont(fontMain);
  cp5.setColorLabel(textColor);

  cp5.addSlider("Month")
    // .setPosition(width-(marginX*6), marginY*1.5)
    .setPosition(width-marginX*5, marginY+20)
    .setSize(300, 30)
    .setRange(0, 11)
    .setNumberOfTickMarks(12)
    .setColorValueLabel(textColor)
    .setColorForeground(tickColor)
    .setColorTickMark(tickColor)  
    .setColorActive(colorHighlight)      
    .setValueLabel(monthNames[0])
    .setDefaultValue(0)
    .setSliderMode(Slider.FLEXIBLE)
    .setColorBackground(bgColor);
    ;
  cp5.addSlider("Year")
    // .setPosition(width-(marginX*6), (marginY*1.5) + 45)
    .setPosition(width-marginX*5, marginY+65)
    .setSize(300, 30)
    .setRange(0, 7)
    .setNumberOfTickMarks(8)
    .setColorTickMark(textColor)
    .setColorValueLabel(textColor)
    .setColorForeground(tickColor)
    .setColorActive(colorHighlight)
    .setColorTickMark(tickColor)
    .setValueLabel(yearNames[0])
    .setSliderMode(Slider.FLEXIBLE)
    .setColorBackground(bgColor)
    .setDefaultValue(0);
    ;
  cp5.addToggle("Sort")
    .setPosition(width-marginX-115,marginY+140)
    .setSize(50,30)
    .setValue(true)
    .setLabel("")
    .setColorForeground(textColor)
    .setColorBackground(tickColor)
    .setColorActive(colorHighlight)
    .setMode(ControlP5.SWITCH)

    ;
  checkbox = cp5.addCheckBox("checkBox")
    .setPosition(marginX-25, marginY+145)
    .setColorForeground(textColor)
    .setColorBackground(tickColor)
    .setColorActive(colorHighlight)
    .setSize(12, 12)
    .setItemsPerRow(1)
    .setSpacingRow(42)
    .addItem("0", 0)
    .addItem("1", 0)
    .addItem("2", 0)
    .addItem("3", 0)
    .addItem("4", 0)
    .addItem("5", 0)
    .addItem("6", 0)
    .addItem("7", 0)
    .addItem("8", 0)
    .addItem("9", 0)
    .activateAll()
    .hideLabels()
    ;

  /*%%%%%% End Setup %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
}

public void controlEvent(ControlEvent theEvent) {
  // println(theEvent.getController().getName());

  if (theEvent.isFrom(cp5.getController("Month"))) {
    selectedMonth = (int)theEvent.getController().getValue();
    theEvent.getController().setValueLabel(monthNames[selectedMonth]);

    float maxAverage = 0;
    float minAverage = MAX_FLOAT;
    for (int i = selectedYear; i < numRows; i+=8) {
      if (maxAverage < averages[i])
        maxAverage = averages[i];
      if (minAverage > averages[i])
        minAverage = averages[i];
    }
    float avg = averages[(selectedMonth * 8) + selectedYear];
    avgColorPick = map(avg, minAverage, maxAverage, 0, 11);
    // println("Color pick: " + round(avgColorPick));
  }

  if (theEvent.isFrom(cp5.getController("Year"))) {
    selectedYear = (int)theEvent.getController().getValue();
    theEvent.getController().setValueLabel(yearNames[selectedYear]);

    float maxAverage = 0;
    float minAverage = MAX_FLOAT;
    for (int i = (selectedYear); i < numRows; i+=8) {
      if (maxAverage < averages[i])
        maxAverage = averages[i];
      if (minAverage > averages[i])
        minAverage = averages[i];
    }
    float avg = averages[(selectedMonth * 8) + (selectedYear)];
    avgColorPick = map(avg, minAverage, maxAverage, 0, 11);
    // println("Color pick: " + round(avgColorPick));
  }  

  if (theEvent.isFrom(checkbox)) {
    // print("got an event from " + checkbox.getName() + "\t\n");
    // println(checkbox.getArrayValue());
    column_picks.clear();
    for (int i = 0; i < numColumns; i++) {
      int n = (int)checkbox.getArrayValue()[i];
      if (n > 0) {
        column_picks.append(i);
      }
    }
    // println("column picks length: " + column_picks.size());
    // print(column_picks);

    for (int i = 0; i < column_picks.size(); i++) {
      // println("new picks: " + column_picks[i]);
    }
    println();
  }

  if (theEvent.isFrom("Sort")) {
    sortSwitch = (int)theEvent.getController().getValue();
  }
}



public void draw()
{
  background(bgColor);
  pickDataDate = (selectedMonth * 8) + selectedYear;
  module(sortedYearMonth, pickDataDate);

  labels();
}
public void keyPressed() {
  if (key == ' ') {
    saveFrame("images/screenshot####.jpg");
    println("saved screenshot");
  }
  // if (keyCode == UP) {
  //   pickDataDate += 1;
  //   if (pickDataDate < 0) pickDataDate = numRows;
  //   if (pickDataDate > numRows-1) pickDataDate = 0;
  // }  
  // if (keyCode == DOWN) {
  //   pickDataDate -= 1;
  //   if (pickDataDate < 0) pickDataDate = numRows;
  //   if (pickDataDate > numRows-1) pickDataDate = 0;
  // }
}
public void labels()
  {
    // DEWEY LIST
    textFont(fontMain);
    textAlign(LEFT);
    for (int q = 0; q < numColumns; q++)
    {
      int labelFill = color(tickColor);
      float current_text_width = (textWidth(deweyNames[q]) + 5) * 1.25f;

      if (hoverTextSwitch == q) {
        labelFill = color(colorHighlight);
        stroke(colorHighlight);

      }
      fill(labelFill);
      textFont(fontHeader2);
      textLeading(14);
      text(deweyNames[q], marginX, marginY + (q * rowSpacing) + 160);
      
      textFont(fontSubtitle);
      text("63" + q, marginX, marginY + (q * rowSpacing) + 145);

      fill(255,0);
      strokeWeight(1.5f);
      rect(marginX - 5, marginY + (q * rowSpacing) + 130, current_text_width, 40);
      noStroke();
    }

    // TITLE
    fill(textColor);
    textFont(fontHeader);
    textAlign(LEFT);
    text("Urban Agriculture in Seattle", marginX, marginY + 40);
    
    textFont(fontHeader2);
    text("Month-by-month popularity reflects seasonal change", 
          marginX, marginY + 58);

    // TOGGLE
    textSize(12);
    textAlign(LEFT,CENTER);
    text("SORT",width-marginX-105,marginY+127);    
    text("DEWEY NUM.",width-marginX-55,marginY+152);
    textAlign(RIGHT,CENTER);
    text("NUM. OF CHECKOUTS",width-marginX-125,marginY+152);


	}
public void legend()
	{

	}
public void module(ArrayList mainList, int mainPick)
{
  noStroke();

  IntList incoming_data = new IntList();
  IntList culled_data = new IntList();
  IntList culled_data_sorted = new IntList();
  IntDict pairedData = new IntDict();
  IntDict paired_dewey_numbers = new IntDict();
  IntDict paired_values = new IntDict(); 

  incoming_data = (IntList)mainList.get(mainPick);

  float avg = 0;
  float avg_picks = 0;
  int sum = 0;
  int sum_picks = 0;
  float max = incoming_data.max();
  float startAngle = -0.8f;

  // create a dictionary to keep track of where the values belong to
  // key = # of checkouts, value = dewey # (-630)
  for (int i = 0; i < numColumns; i++) {
    pairedData.set(str(i), incoming_data.get(i));
    sum += incoming_data.get(i);
  }

  // cull data as per user definition
  for (int i = 0; i < column_picks.size(); i++) {
    String the_data = str(column_picks.get(i));
    culled_data.append(pairedData.get(the_data));
    culled_data_sorted.append(pairedData.get(the_data));
    sum_picks += pairedData.get(the_data);
    paired_dewey_numbers.set(str(pairedData.get(the_data)), column_picks.get(i));
  }
  // println(paired_dewey_numbers);

  // make a dictionary where key = SORTED checkout val, value = dewey # 
  culled_data_sorted.sortReverse();
  for (int i = 0; i < column_picks.size(); i++) {
    paired_values.set(str(culled_data_sorted.get(i)), i);
  }

  avg = sum/numColumns;
  avg_picks = sum_picks/column_picks.size();

  // sort the data or not
  if (sortSwitch == 1) culled_data.sortReverse();

  
  // label switch
  int labelIndex = -1;
  // int value_index = 0;
  int hover_index = 0;

  for (int i = 0; i < column_picks.size(); i++)
  {
    float this_data = culled_data.get(i);
    float this_data_sorted = culled_data_sorted.get(i);
    float maxRatio = this_data/culled_data.max();
    float sumRatio = this_data/sum_picks;
    float newVal = this_data*360/sum_picks;
    float endAngle = startAngle + radians(newVal);
    float newSize = chartRadius + (this_data*300)/sum_picks;
    
    int sorted_index = 0;
    int value_index = round(this_data);


    // get mouse position relative to sketch center
    float translateX = map(mouseX, 0, width, -width/2, width/2);
    float translateY = map(mouseY, 0, height, -height/2, height/2);

    // cartesian to polar coordinates
    float mouseRad = sqrt(pow(translateY, 2) + pow(translateX, 2));
    float mouseAngle = atan2(translateY, translateX);

    // fix for full circle
    if (mouseAngle < 0) {
      mouseAngle = PI + (PI + mouseAngle);
    }

    // mouse interaction
    colorMode(HSB);
    float fillColor = 255 * maxRatio;
    float fillAlpha = (fillColor*20)+20;
    if (((mouseAngle > startAngle) && (mouseAngle < endAngle)) 
          && (mouseRad < newSize * .5f) && (mouseRad > innerCircleRadius * .5f)) {
      // mouse over segment
      labelIndex = round(this_data);
      sorted_index = round(this_data_sorted);
      hover_index = paired_dewey_numbers.get(str(labelIndex));
      hoverTextSwitch = hover_index;
      if (sortSwitch == 1) fill(colorMapSelected[paired_values.get(str(sorted_index))]);
      if (sortSwitch == 0) fill(colorMapSelected[paired_values.get(str(value_index))]);

    }
    // fix for negative start angle values
    else if ( (startAngle < 0) && (mouseAngle > startAngle+(2*PI)) && (mouseAngle < 2*PI) 
                  && (mouseRad < newSize * .5f) && (mouseRad > innerCircleRadius * .5f))  {
      labelIndex = round(this_data);
      sorted_index = round(this_data_sorted);
      hover_index = paired_dewey_numbers.get(str(labelIndex));
      hoverTextSwitch = hover_index;
      if (sortSwitch == 1) fill(colorMapSelected[paired_values.get(str(sorted_index))]);
      if (sortSwitch == 0) fill(colorMapSelected[paired_values.get(str(value_index))]);

    } else {
      // mouse not over segment
      if (sortSwitch == 1) fill(colorMap[i]);
      if (sortSwitch == 0) fill(colorMap[paired_values.get(str(value_index))]);

    }
    colorMode(RGB);


    // draw the chart
    arc(width/2, height/2, newSize, newSize, startAngle, endAngle);
    
    // draw another partially transparent chart on top so colors change with month
    fill(colorMapMonth[round(avgColorPick)],55);
    arc(width/2, height/2, newSize, newSize, startAngle, endAngle);
    
    startAngle = endAngle;

    fill(bgColor);
    ellipse(width/2, height/2, innerCircleRadius, innerCircleRadius);
    
    // draw the label
    if (labelIndex != -1) {               
      String txt = str(labelIndex);
      float labelW = textWidth(txt);
      fill(tickColor);
      textFont(fontCenter);
      textAlign(CENTER,CENTER);
      text(txt, width/2, (height/2)-10);
    }
  
  }
}

public void test()
{
  int[ ] coords = {
    40, 40, 80, 60, 100, 100, 60, 120, 50, 150
  };
  int i;

  noFill();
  stroke(0);
  beginShape();
  curveVertex(40, 40); // the first control point
  curveVertex(40, 40); // is also the start point of curve
  curveVertex(80, 60);
  curveVertex(100, 100);
  curveVertex(60, 120);
  curveVertex(50, 150); // the last point of curve
  curveVertex(50, 150); // is also the last control point
  endShape();
  
  fill(255, 0, 0);
  noStroke();
  for (i = 0; i < coords.length; i += 2)
  {
    ellipse(coords[i], coords[i + 1], 3, 3);
  }
  
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "hw2_reorderable" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
