/*  Urban Agriculture in Seattle with Polar Area Diagram
    Kurt Kaminski
    Algorithmic Visulization, Prof. George Legrady
    MAT 2015
*/

import controlP5.*;

ControlP5 cp5;
CheckBox checkbox;

int chartRadius = 440;
float innerCircleRadius = chartRadius*.33;
int xPosition = 20;
int pos = 0;

int pickDataDate = 0;
int hoverTextSwitch = -1;
int sortSwitch = 1;

float marginX = 90;
float marginY = 40;    
int [][] dataMatrix = null;
float [] averages = new float[96];
float avgColorPick = 0;
int maxCount;
int numRows, numColumns, rowSpacing;

ArrayList<IntList> sortedYearMonth = new ArrayList<IntList>();  
IntList column_picks; // user selection of column data

String [] deweyNames = new String[10];
String [] monthNames = {  "January", "February", "March", "April", "May", "June", "July", "August",
                          "September", "October", "November", "December"};
String [] yearNames = {"2006","2007","2008","2009","2010","2011","2012","2013"};
PFont fontHeader = createFont("SukhumvitSet-SemiBold", 30, true);
PFont fontHeader2 = createFont("SourceSansPro-Semibold", 14, true);
PFont fontCenter = createFont("AvenirNext-UltraLightItalic", 48, true);
PFont fontHover = createFont("AvenirNext-UltraLight", 32, true);
PFont fontMain = createFont("SourceSansPro-Semibold", 12, true);
PFont fontSubtitle = createFont("Avenir-Regular", 11, true);

color bgColor = color(222,221,217);
color textColor = color(84, 81, 109);
color tickColor = color(151,151,184);
color colorHighlight = #4B7A26;
color [] colorMap = {#A7F58C,#7CDE91,#58C593,#3CAC91,#2B9289,#27787D,#29606C,#2A4857,#263141,#1D1D29};
color [] colorMapSelected = {#105C65,#0E6C73,#0B7C80,#078D8D,#059E99,#08AFA5,#11C1B0,#1DD2BA,#2BE4C3,#3BF7CB};
color [] colorMapMonth = {#823B21,#8B471D,#925518,#976311,#99720B,#998208,#95920F,#8FA21D,#85B22F,#76C244,#62D25C,#40E177};

int sliderValue = 100;
int sliderTicks1 = 100;
int sliderTicks2 = 30;
int selectedMonth = 1;
int selectedYear = 1;
Slider abc;

Table myTable;

void setup()
{
  size(1200, 900);
  smooth();
  println("setup done");

  rowSpacing = 54;
  myTable = loadTable("hw1_yearmonth.csv", "header");
  numRows = myTable.getRowCount();
  numColumns = myTable.getColumnCount() - 2;
  dataMatrix = new int[numRows][numColumns];
  maxCount = dataMatrix[0][0];

  for (int i = 0; i < numRows; i++)
  {
    for (int j = 0; j < numColumns; j++) //skipping two columns j=2
    {
      dataMatrix[i][j] = myTable.getInt(i, (j+2));
      // print(dataMatrix[i][j] + " " );
      //            print(i + "," + j + " " );

      if (dataMatrix[i][j] > maxCount)
        maxCount = dataMatrix[i][j];
    }
    // println();
  }

  println("Rows: " + numRows + ", Columns: " + numColumns + ", max: " + maxCount);
  println();
  println();

  deweyNames[0] = "Agriculture & related technology";
  deweyNames[1] = "Apparatus, equipment, materials";
  deweyNames[2] = "Plant injuries, diseases, pests";
  deweyNames[3] = "Field & plantation crops";
  deweyNames[4] = "Orchards, fruits, forestry";
  deweyNames[5] = "Garden crops, horticulture";
  deweyNames[6] = "Animal husbandry";
  deweyNames[7] = "Processing dairy";
  deweyNames[8] = "Insect culture";
  deweyNames[9] = "Hunting, fishing, conservation";
 
 
  ///////////////////////////////////////////////////////////////////////
  ///////// Load arrays with data ///////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////

  background(bgColor);
  noStroke();
  
  int [] sorted = null;
  
  // Create sorted lists for each year/month
  for (int i = 0; i < numRows; i++) {
    IntList data = new IntList();
    for (int j = 0; j < numColumns; j++) {
      data.append(dataMatrix[i][j]);
    }
    sortedYearMonth.add(data);
  }

  // calculate all monthly averages
  for (int i = 0; i < numRows; i++) {
    float avg = 0;  
    IntList getAvg = sortedYearMonth.get(i);
      for (int j = 0; j < numColumns; j++) {
        avg += getAvg.get(j);
      }
    averages[i] = avg/10;
//    println(averages[i]);
  }

  // load dewey class visibility so its on by default
  column_picks = new IntList();
  for (int i = 0; i < numColumns; i++) {
    column_picks.append(i);
  }
  
  ///////////////////////////////////////////////////////////////////////
  ///////// Sliders /////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////
  cp5 = new ControlP5(this);
  cp5.setControlFont(fontMain);
  cp5.setColorLabel(textColor);

  cp5.addSlider("Month")
    // .setPosition(width-(marginX*6), marginY*1.5)
    .setPosition(width-marginX*5, marginY+20)
    .setSize(300, 30)
    .setRange(0, 11)
    .setNumberOfTickMarks(12)
    .setColorValueLabel(textColor)
    .setColorForeground(tickColor)
    .setColorTickMark(tickColor)  
    .setColorActive(colorHighlight)      
    .setValueLabel(monthNames[0])
    .setDefaultValue(0)
    .setSliderMode(Slider.FLEXIBLE)
    .setColorBackground(bgColor);
    ;
  cp5.addSlider("Year")
    // .setPosition(width-(marginX*6), (marginY*1.5) + 45)
    .setPosition(width-marginX*5, marginY+65)
    .setSize(300, 30)
    .setRange(0, 7)
    .setNumberOfTickMarks(8)
    .setColorTickMark(textColor)
    .setColorValueLabel(textColor)
    .setColorForeground(tickColor)
    .setColorActive(colorHighlight)
    .setColorTickMark(tickColor)
    .setValueLabel(yearNames[0])
    .setSliderMode(Slider.FLEXIBLE)
    .setColorBackground(bgColor)
    .setDefaultValue(0);
    ;
  cp5.addToggle("Sort")
    .setPosition(width-marginX-115,marginY+140)
    .setSize(50,30)
    .setValue(true)
    .setLabel("")
    .setColorForeground(textColor)
    .setColorBackground(tickColor)
    .setColorActive(colorHighlight)
    .setMode(ControlP5.SWITCH)

    ;
  checkbox = cp5.addCheckBox("checkBox")
    .setPosition(marginX-25, marginY+145)
    .setColorForeground(textColor)
    .setColorBackground(tickColor)
    .setColorActive(colorHighlight)
    .setSize(12, 12)
    .setItemsPerRow(1)
    .setSpacingRow(42)
    .addItem("0", 0)
    .addItem("1", 0)
    .addItem("2", 0)
    .addItem("3", 0)
    .addItem("4", 0)
    .addItem("5", 0)
    .addItem("6", 0)
    .addItem("7", 0)
    .addItem("8", 0)
    .addItem("9", 0)
    .activateAll()
    .hideLabels()
    ;

  /*%%%%%% End Setup %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
}

void controlEvent(ControlEvent theEvent) {
  // println(theEvent.getController().getName());

  if (theEvent.isFrom(cp5.getController("Month"))) {
    selectedMonth = (int)theEvent.getController().getValue();
    theEvent.getController().setValueLabel(monthNames[selectedMonth]);

    float maxAverage = 0;
    float minAverage = MAX_FLOAT;
    for (int i = selectedYear; i < numRows; i+=8) {
      if (maxAverage < averages[i])
        maxAverage = averages[i];
      if (minAverage > averages[i])
        minAverage = averages[i];
    }
    float avg = averages[(selectedMonth * 8) + selectedYear];
    avgColorPick = map(avg, minAverage, maxAverage, 0, 11);
    // println("Color pick: " + round(avgColorPick));
  }

  if (theEvent.isFrom(cp5.getController("Year"))) {
    selectedYear = (int)theEvent.getController().getValue();
    theEvent.getController().setValueLabel(yearNames[selectedYear]);

    float maxAverage = 0;
    float minAverage = MAX_FLOAT;
    for (int i = (selectedYear); i < numRows; i+=8) {
      if (maxAverage < averages[i])
        maxAverage = averages[i];
      if (minAverage > averages[i])
        minAverage = averages[i];
    }
    float avg = averages[(selectedMonth * 8) + (selectedYear)];
    avgColorPick = map(avg, minAverage, maxAverage, 0, 11);
    // println("Color pick: " + round(avgColorPick));
  }  

  if (theEvent.isFrom(checkbox)) {
    // print("got an event from " + checkbox.getName() + "\t\n");
    // println(checkbox.getArrayValue());
    column_picks.clear();
    for (int i = 0; i < numColumns; i++) {
      int n = (int)checkbox.getArrayValue()[i];
      if (n > 0) {
        column_picks.append(i);
      }
    }
    // println("column picks length: " + column_picks.size());
    // print(column_picks);

    for (int i = 0; i < column_picks.size(); i++) {
      // println("new picks: " + column_picks[i]);
    }
    println();
  }

  if (theEvent.isFrom("Sort")) {
    sortSwitch = (int)theEvent.getController().getValue();
  }
}



